// Copyright 2017 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#include "google/cloud/internal/build_info.h"
#include "google/cloud/internal/compiler_info.h"
#include <algorithm>
#include <cctype>

namespace google {
namespace cloud {
inline namespace GOOGLE_CLOUD_CPP_NS {
namespace internal {

// NOLINTNEXTLINE(readability-identifier-naming)
std::string compiler() {
  return CompilerId() + " " + CompilerVersion();
}

// NOLINTNEXTLINE(readability-identifier-naming)
std::string compiler_flags() {
  static char const kCompilerFlags[] =
      R"""( -O2 -fwrapv -pipe -Wformat -Werror=format-security -gdwarf-4 -fno-debug-types-section -fvar-tracking-assignments -g2 -Wp,-D_FORTIFY_SOURCE=2 -fstack-protector-strong --param=ssp-buffer-size=4 -Werror=trampolines -fPIC -mtune=generic -march=x86-64 -gdwarf-4 -fno-debug-types-section -fvar-tracking-assignments -g2  -DNDEBUG -DQT_NO_DEBUG)""";
  return kCompilerFlags;
}

// NOLINTNEXTLINE(readability-identifier-naming)
std::string build_metadata() {
  // Sometimes GOOGLE_CLOUD_CPP_BUILD_METADATA string expands to nothing, and
  // then clang-tidy complains.
  // NOLINTNEXTLINE(readability-redundant-string-init)
  static char const kBuildMetadata[] = R"""()""";
  return kBuildMetadata;
}

}  // namespace internal
}  // namespace GOOGLE_CLOUD_CPP_NS
}  // namespace cloud
}  // namespace google
