// Copyright 2022 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/cloud/bigquery/datapolicies/v1/datapolicy.proto

#include "google/cloud/bigquery/datapolicies/v1/internal/data_policy_connection_impl.h"
#include "google/cloud/bigquery/datapolicies/v1/internal/data_policy_option_defaults.h"
#include "google/cloud/background_threads.h"
#include "google/cloud/common_options.h"
#include "google/cloud/grpc_options.h"
#include "google/cloud/internal/pagination_range.h"
#include "google/cloud/internal/retry_loop.h"
#include <memory>
#include <utility>

namespace google {
namespace cloud {
namespace bigquery_datapolicies_v1_internal {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
namespace {

std::unique_ptr<bigquery_datapolicies_v1::DataPolicyServiceRetryPolicy>
retry_policy(Options const& options) {
  return options
      .get<bigquery_datapolicies_v1::DataPolicyServiceRetryPolicyOption>()
      ->clone();
}

std::unique_ptr<BackoffPolicy> backoff_policy(Options const& options) {
  return options
      .get<bigquery_datapolicies_v1::DataPolicyServiceBackoffPolicyOption>()
      ->clone();
}

std::unique_ptr<
    bigquery_datapolicies_v1::DataPolicyServiceConnectionIdempotencyPolicy>
idempotency_policy(Options const& options) {
  return options
      .get<bigquery_datapolicies_v1::
               DataPolicyServiceConnectionIdempotencyPolicyOption>()
      ->clone();
}

}  // namespace

DataPolicyServiceConnectionImpl::DataPolicyServiceConnectionImpl(
    std::unique_ptr<google::cloud::BackgroundThreads> background,
    std::shared_ptr<bigquery_datapolicies_v1_internal::DataPolicyServiceStub>
        stub,
    Options options)
    : background_(std::move(background)),
      stub_(std::move(stub)),
      options_(internal::MergeOptions(
          std::move(options), DataPolicyServiceConnection::options())) {}

StatusOr<google::cloud::bigquery::datapolicies::v1::DataPolicy>
DataPolicyServiceConnectionImpl::CreateDataPolicy(
    google::cloud::bigquery::datapolicies::v1::CreateDataPolicyRequest const&
        request) {
  auto current = google::cloud::internal::SaveCurrentOptions();
  return google::cloud::internal::RetryLoop(
      retry_policy(*current), backoff_policy(*current),
      idempotency_policy(*current)->CreateDataPolicy(request),
      [this](grpc::ClientContext& context, Options const& options,
             google::cloud::bigquery::datapolicies::v1::
                 CreateDataPolicyRequest const& request) {
        return stub_->CreateDataPolicy(context, options, request);
      },
      *current, request, __func__);
}

StatusOr<google::cloud::bigquery::datapolicies::v1::DataPolicy>
DataPolicyServiceConnectionImpl::UpdateDataPolicy(
    google::cloud::bigquery::datapolicies::v1::UpdateDataPolicyRequest const&
        request) {
  auto current = google::cloud::internal::SaveCurrentOptions();
  return google::cloud::internal::RetryLoop(
      retry_policy(*current), backoff_policy(*current),
      idempotency_policy(*current)->UpdateDataPolicy(request),
      [this](grpc::ClientContext& context, Options const& options,
             google::cloud::bigquery::datapolicies::v1::
                 UpdateDataPolicyRequest const& request) {
        return stub_->UpdateDataPolicy(context, options, request);
      },
      *current, request, __func__);
}

StatusOr<google::cloud::bigquery::datapolicies::v1::DataPolicy>
DataPolicyServiceConnectionImpl::RenameDataPolicy(
    google::cloud::bigquery::datapolicies::v1::RenameDataPolicyRequest const&
        request) {
  auto current = google::cloud::internal::SaveCurrentOptions();
  return google::cloud::internal::RetryLoop(
      retry_policy(*current), backoff_policy(*current),
      idempotency_policy(*current)->RenameDataPolicy(request),
      [this](grpc::ClientContext& context, Options const& options,
             google::cloud::bigquery::datapolicies::v1::
                 RenameDataPolicyRequest const& request) {
        return stub_->RenameDataPolicy(context, options, request);
      },
      *current, request, __func__);
}

Status DataPolicyServiceConnectionImpl::DeleteDataPolicy(
    google::cloud::bigquery::datapolicies::v1::DeleteDataPolicyRequest const&
        request) {
  auto current = google::cloud::internal::SaveCurrentOptions();
  return google::cloud::internal::RetryLoop(
      retry_policy(*current), backoff_policy(*current),
      idempotency_policy(*current)->DeleteDataPolicy(request),
      [this](grpc::ClientContext& context, Options const& options,
             google::cloud::bigquery::datapolicies::v1::
                 DeleteDataPolicyRequest const& request) {
        return stub_->DeleteDataPolicy(context, options, request);
      },
      *current, request, __func__);
}

StatusOr<google::cloud::bigquery::datapolicies::v1::DataPolicy>
DataPolicyServiceConnectionImpl::GetDataPolicy(
    google::cloud::bigquery::datapolicies::v1::GetDataPolicyRequest const&
        request) {
  auto current = google::cloud::internal::SaveCurrentOptions();
  return google::cloud::internal::RetryLoop(
      retry_policy(*current), backoff_policy(*current),
      idempotency_policy(*current)->GetDataPolicy(request),
      [this](
          grpc::ClientContext& context, Options const& options,
          google::cloud::bigquery::datapolicies::v1::GetDataPolicyRequest const&
              request) {
        return stub_->GetDataPolicy(context, options, request);
      },
      *current, request, __func__);
}

StreamRange<google::cloud::bigquery::datapolicies::v1::DataPolicy>
DataPolicyServiceConnectionImpl::ListDataPolicies(
    google::cloud::bigquery::datapolicies::v1::ListDataPoliciesRequest
        request) {
  request.clear_page_token();
  auto current = google::cloud::internal::SaveCurrentOptions();
  auto idempotency = idempotency_policy(*current)->ListDataPolicies(request);
  char const* function_name = __func__;
  return google::cloud::internal::MakePaginationRange<
      StreamRange<google::cloud::bigquery::datapolicies::v1::DataPolicy>>(
      current, std::move(request),
      [idempotency, function_name, stub = stub_,
       retry = std::shared_ptr<
           bigquery_datapolicies_v1::DataPolicyServiceRetryPolicy>(
           retry_policy(*current)),
       backoff = std::shared_ptr<BackoffPolicy>(backoff_policy(*current))](
          Options const& options, google::cloud::bigquery::datapolicies::v1::
                                      ListDataPoliciesRequest const& r) {
        return google::cloud::internal::RetryLoop(
            retry->clone(), backoff->clone(), idempotency,
            [stub](grpc::ClientContext& context, Options const& options,
                   google::cloud::bigquery::datapolicies::v1::
                       ListDataPoliciesRequest const& request) {
              return stub->ListDataPolicies(context, options, request);
            },
            options, r, function_name);
      },
      [](google::cloud::bigquery::datapolicies::v1::ListDataPoliciesResponse
             r) {
        std::vector<google::cloud::bigquery::datapolicies::v1::DataPolicy>
            result(r.data_policies().size());
        auto& messages = *r.mutable_data_policies();
        std::move(messages.begin(), messages.end(), result.begin());
        return result;
      });
}

StatusOr<google::iam::v1::Policy> DataPolicyServiceConnectionImpl::GetIamPolicy(
    google::iam::v1::GetIamPolicyRequest const& request) {
  auto current = google::cloud::internal::SaveCurrentOptions();
  return google::cloud::internal::RetryLoop(
      retry_policy(*current), backoff_policy(*current),
      idempotency_policy(*current)->GetIamPolicy(request),
      [this](grpc::ClientContext& context, Options const& options,
             google::iam::v1::GetIamPolicyRequest const& request) {
        return stub_->GetIamPolicy(context, options, request);
      },
      *current, request, __func__);
}

StatusOr<google::iam::v1::Policy> DataPolicyServiceConnectionImpl::SetIamPolicy(
    google::iam::v1::SetIamPolicyRequest const& request) {
  auto current = google::cloud::internal::SaveCurrentOptions();
  return google::cloud::internal::RetryLoop(
      retry_policy(*current), backoff_policy(*current),
      idempotency_policy(*current)->SetIamPolicy(request),
      [this](grpc::ClientContext& context, Options const& options,
             google::iam::v1::SetIamPolicyRequest const& request) {
        return stub_->SetIamPolicy(context, options, request);
      },
      *current, request, __func__);
}

StatusOr<google::iam::v1::TestIamPermissionsResponse>
DataPolicyServiceConnectionImpl::TestIamPermissions(
    google::iam::v1::TestIamPermissionsRequest const& request) {
  auto current = google::cloud::internal::SaveCurrentOptions();
  return google::cloud::internal::RetryLoop(
      retry_policy(*current), backoff_policy(*current),
      idempotency_policy(*current)->TestIamPermissions(request),
      [this](grpc::ClientContext& context, Options const& options,
             google::iam::v1::TestIamPermissionsRequest const& request) {
        return stub_->TestIamPermissions(context, options, request);
      },
      *current, request, __func__);
}

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace bigquery_datapolicies_v1_internal
}  // namespace cloud
}  // namespace google
